import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:intl/intl.dart';
import 'package:syncrow_web/pages/visitor_password/bloc/visitor_password_event.dart';
import 'package:syncrow_web/pages/visitor_password/bloc/visitor_password_state.dart';
import 'package:syncrow_web/pages/visitor_password/model/device_model.dart';
import 'package:syncrow_web/pages/visitor_password/model/schedule_model.dart';
import 'package:syncrow_web/services/access_mang_api.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/snack_bar.dart';
  List<String> selectedDevices = [];

// Define the BLoC
class VisitorPasswordBloc
    extends Bloc<VisitorPasswordEvent, VisitorPasswordState> {
  VisitorPasswordBloc() : super(VisitorPasswordInitial()) {
    on<SelectUsageFrequency>(selectUsageFrequency);
    on<FetchDevice>(_onFetchDevice);
    on<SelectPasswordType>(selectAccessType);
    on<SelectTimeVisitorPassword>(selectTimeVisitorPassword);
    on<ToggleRepeatEvent>(toggleRepeat);
    on<ToggleDaySelectionEvent>(toggleDaySelection);
    on<SelectDeviceEvent>(selectDevice);
    on<UpdateFilteredDevicesEvent>(_onUpdateFilteredDevices);

    on<OnlineOneTimePasswordEvent>(postOnlineOneTimePassword);
    on<OnlineMultipleTimePasswordEvent>(postOnlineMultipleTimePassword);

    on<OfflineMultipleTimePasswordEvent>(postOfflineMultipleTimePassword);
    on<OfflineOneTimePasswordEvent>(postOfflineOneTimePassword);

  }
  final TextEditingController userNameController = TextEditingController();
  final TextEditingController emailController = TextEditingController();

  final TextEditingController deviceNameController = TextEditingController();
  final TextEditingController deviceIdController = TextEditingController();
  final TextEditingController unitNameController = TextEditingController();
  final TextEditingController virtualAddressController = TextEditingController();



  List<DeviceModel> data = [];
  List<String> selectedDeviceIds = [];



  final forgetFormKey = GlobalKey<FormState>();

  String accessTypeSelected = 'Online Password';
  String usageFrequencySelected = 'One-Time';
  String passwordController = '';

  bool repeat = false;

  int? effectiveTimeTimeStamp;
  int? expirationTimeTimeStamp;

  int? repeatEffectiveTimeTimeStamp;
  int? repeatExpirationTimeTimeStamp;

  String startTime = 'Start Time';
  String endTime = 'End Time';


  String repeatStartTime = 'Start Time';
  String repeatEndTime = 'End Time';

  // DateTime? repeatStartTime=DateTime.now();
  // DateTime? repeatEndTime;

  selectAccessType(
      SelectPasswordType event, Emitter<VisitorPasswordState> emit) {
    accessTypeSelected = event.type;
    emit(PasswordTypeSelected(event.type));
  }

  selectUsageFrequency(
      SelectUsageFrequency event, Emitter<VisitorPasswordState> emit) {
    usageFrequencySelected = event.usageType;
    emit(UsageFrequencySelected(event.usageType));
  }

  Future<void> selectTimeVisitorPassword(
      SelectTimeVisitorPassword event,
      Emitter<VisitorPasswordState> emit) async {
    final DateTime? picked = await showDatePicker(
      context: event.context,
      initialDate: DateTime.now(),
      firstDate: DateTime(2015, 8),
      lastDate: DateTime(3101),
    );
    if (picked != null) {
      final TimeOfDay? timePicked = await showTimePicker(
        context: event.context,
        initialTime: TimeOfDay.now(),
        builder: (context, child) {
          return Theme(
            data: ThemeData.light().copyWith(
              colorScheme: const ColorScheme.light(
                primary: ColorsManager.primaryColor,
                onSurface: Colors.black,
              ),
              buttonTheme: const ButtonThemeData(
                colorScheme: ColorScheme.light(
                  primary: Colors.green,
                ),
              ),
            ),
            child: child!,
          );
        },
      );
      if (timePicked != null) {
        final selectedDateTime = DateTime(
          picked.year,
          picked.month,
          picked.day,
          timePicked.hour,
          timePicked.minute,
        );
        final selectedTimestamp = DateTime(
              selectedDateTime.year,
              selectedDateTime.month,
              selectedDateTime.day,
              selectedDateTime.hour,
              selectedDateTime.minute,
            ).millisecondsSinceEpoch ~/
            1000; // Divide by 1000 to remove milliseconds
        if (event.isStart) {
          if (expirationTimeTimeStamp != null && selectedTimestamp >expirationTimeTimeStamp!) {
            CustomSnackBar.displaySnackBar('Effective Time cannot be later than Expiration Time.');
          } else {
            if(event.isRepeat==true)
            {repeatStartTime  = selectedDateTime.toString().split('.').first;}

            else // Remove seconds and milliseconds
            {startTime = selectedDateTime.toString().split('.').first;}
            effectiveTimeTimeStamp = selectedTimestamp;
            emit(ChangeTimeState());
          }
          emit(ChangeTimeState());
        } else {
          if (effectiveTimeTimeStamp != null &&
              selectedTimestamp < effectiveTimeTimeStamp!) {
            CustomSnackBar.displaySnackBar('Expiration Time cannot be earlier than Effective Time.');
          } else {
            if(event.isRepeat==true)
            {repeatEndTime  = selectedDateTime.toString().split('.').first;}
            else
            {endTime = selectedDateTime.toString().split('.').first;}
            expirationTimeTimeStamp = selectedTimestamp;
            emit(ChangeTimeState());

          }
          emit(ChangeTimeState());

          emit(VisitorPasswordInitial());
        }
      }
    }

    // emit(AccessInitial());
    // emit(TableLoaded(data));
  }

  bool toggleRepeat(
      ToggleRepeatEvent event, Emitter<VisitorPasswordState> emit) {
    emit(LoadingInitialState());
    repeat = !repeat;
    emit(IsRepeatState(repeat: repeat));
    return repeat;
  }

  List<Map<String, String>> days = [
    {"day": "Sun", "key": "Sun"},
    {"day": "Mon", "key": "Mon"},
    {"day": "Tue", "key": "Tue"},
    {"day": "Wed", "key": "Wed"},
    {"day": "Thu", "key": "Thu"},
    {"day": "Fri", "key": "Fri"},
    {"day": "Sat", "key": "Sat"},
  ];

  List<String> selectedDays = [];

  Future<void> toggleDaySelection(
    ToggleDaySelectionEvent event,
    Emitter<VisitorPasswordState> emit,
  ) async {
    emit(LoadingInitialState());
    if (selectedDays.contains(event.key)) {
      selectedDays.remove(event.key);
    } else {
      selectedDays.add(event.key);
    }
    emit(ChangeTimeState());
  }

  Future<void> _onFetchDevice(
      FetchDevice event, Emitter<VisitorPasswordState> emit) async {
    try {
      emit(DeviceLoaded());
      data = await AccessMangApi().fetchDevices();
      emit(TableLoaded(data));
    } catch (e) {
      emit(FailedState(e.toString()));
    }
  }

  //online password

  Future<void> postOnlineOneTimePassword(
      OnlineOneTimePasswordEvent event,
      Emitter<VisitorPasswordState> emit) async {
    try {
      print('selectedDevices$selectedDevices');
      // emit(DeviceLoaded());
      await AccessMangApi().postOnlineOneTime(
          email: event.email,
          devicesUuid: selectedDevices,
          passwordName: event.passwordName);
      // emit(TableLoaded(data));
    } catch (e) {
      emit(FailedState(e.toString()));
    }
  }

  Future<void> postOnlineMultipleTimePassword(
      OnlineMultipleTimePasswordEvent event,
      Emitter<VisitorPasswordState> emit) async {
    try {
      generate7DigitNumber();
      // emit(DeviceLoaded());
      await AccessMangApi().postOnlineMultipleTime(
        scheduleList:[
          // if (repeat)
          //   Schedule(
          //     effectiveTime: getTimeOnly(repeatStartTime),
          //     invalidTime: getTimeOnly(repeatEndTime).toString(),
          //     workingDay: selectedDays,
          //   ),
        ] ,
          password: passwordController,
          invalidTime:event.invalidTime ,
          effectiveTime:event.effectiveTime ,
          email: event.email,
          devicesUuid: selectedDevices,
          passwordName: event.passwordName
      );
      // emit(TableLoaded(data));
    } catch (e) {
      emit(FailedState(e.toString()));
    }
  }


  //offline password
  Future<void> postOfflineOneTimePassword(
      OfflineOneTimePasswordEvent event,
      Emitter<VisitorPasswordState> emit) async {
    try {
      generate7DigitNumber();
      // emit(DeviceLoaded());
      await AccessMangApi().postOffLineOneTime(
          email: event.email,
          devicesUuid: selectedDevices,
          passwordName: event.passwordName
      );
      // emit(TableLoaded(data));
    } catch (e) {
      emit(FailedState(e.toString()));
    }
  }

  Future<void> postOfflineMultipleTimePassword(
      OfflineMultipleTimePasswordEvent event,
      Emitter<VisitorPasswordState> emit) async {
    try {
      generate7DigitNumber();
      // emit(DeviceLoaded());
      await AccessMangApi().postOffLineMultipleTime(
          email: event.email,
          devicesUuid: selectedDevices,
          passwordName: event.passwordName,
          invalidTime:event.invalidTime ,
          effectiveTime:event.effectiveTime
      );
      // emit(TableLoaded(data));
    } catch (e) {
      emit(FailedState(e.toString()));
    }
  }


  void selectDevice(
      SelectDeviceEvent event, Emitter<VisitorPasswordState> emit) {
    if (selectedDeviceIds.contains(event.deviceId)) {
      selectedDeviceIds.remove(event.deviceId);
    } else {
      selectedDeviceIds.add(event.deviceId);
    }
    selectedDevices=selectedDeviceIds;
    print(selectedDevices);
  }

  String? validate(String? value) {
    if (value == null || value.isEmpty) {
      return '';
    }
    return null;
  }


  Future generate7DigitNumber() async {
    emit(LoadingInitialState());
    passwordController='';
    Random random = Random();
    int min = 1000000;
    int max = 9999999;
    passwordController = (min + random.nextInt(max - min + 1)).toString();
    emit(GeneratePasswordState());
    return passwordController;
  }
  String getTimeOnly(DateTime? dateTime) {
    if (dateTime == null) return '';
    return DateFormat('HH:mm').format(dateTime);
  }


  void filterDevices() {
    final deviceName = deviceNameController.text.toLowerCase();
    final deviceId = deviceIdController.text.toLowerCase();
    final unitName = unitNameController.text.toLowerCase();

    final filteredData = data.where((device) {
      final matchesDeviceName = device.name.toLowerCase().contains(deviceName);
      final matchesDeviceId = device.uuid.toLowerCase().contains(deviceId);
      // final matchesUnitName = device.unitName.toLowerCase().contains(unitName); // Assuming unitName is a property of the device

      return matchesDeviceName && matchesDeviceId ;
    }).toList();
    // emit(TableLoaded(filteredData));

     add(UpdateFilteredDevicesEvent(filteredData));
  }
  @override
  Stream<VisitorPasswordState> mapEventToState(VisitorPasswordEvent event) async* {
    if (event is FetchDevice) {
      // Fetching logic...
    } else if (event is UpdateFilteredDevicesEvent) {
      yield TableLoaded(event.filteredData);
    }
  }

  void _onUpdateFilteredDevices(UpdateFilteredDevicesEvent event, Emitter<VisitorPasswordState> emit) {
    emit(TableLoaded(event.filteredData));
  }
}
